/*
 * LineBreak  ~  v.0.2  ~  NightOwl  ~  Scoreboard code.
 */
import java.awt.*;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Scanner;

public class Scores {
    private Iterator it;
    private List<LevelRecord> table;
    private LevelRecord pointer;
    private Font first, second;
    private FontMetrics fmetr, smetr;
    private ListIterator currit;
    private LevelRecord current;
    private Rectangle leftRect, rightRect;
    private boolean left = false, right = false;

    public Scores(String filename){
        table = new ArrayList<LevelRecord>();
        loadScores(filename);
        currit = table.listIterator();
        current = (LevelRecord)currit.next();
    }
    public void addRecord(String name){
        table.add(new LevelRecord(name));
    }
    public void addItem(String levelname, String name, int score){
        it = table.iterator();
        while(it.hasNext()){
            pointer = (LevelRecord)it.next();
            if(pointer.levelname.equals(levelname)){
                pointer.addItem(name,score);
                return;
            }
        }
        addRecord(levelname);
        addItem(levelname, name, score);
    }

    // set
    public void setFonts(Font first, FontMetrics firstmetr, Font second, FontMetrics secondmetr){
        this.first = first; this.fmetr = firstmetr;
        this.second = second; this.smetr = secondmetr;
        // calculate rect bounds
        leftRect = new Rectangle(Const.WIDTH/2-smetr.stringWidth(current.levelname)/2-50, 86, 30, smetr.getHeight());
        rightRect = new Rectangle(Const.WIDTH/2+smetr.stringWidth(current.levelname)/2+20, 86, 30, smetr.getHeight());
    }
    public void setCurrent(String name){
        for(LevelRecord rec: table){
            if(rec.levelname.equals(name)){
                current = rec;
                return;
            }
        }
    }

    // get
    public boolean exists(String name){
        it = table.iterator();
        while(it.hasNext()){
            pointer = (LevelRecord)it.next();
            if(pointer.levelname.equals(name)){
                return true;
            }
        }
        return false;
    }
    public boolean leftSelected(){
        return left;
    }
    public boolean rightSelected(){
        return right;
    }

    // classes
    private class LevelRecord{
        protected String levelname;
        protected Item[] record = new Item[10];

        public LevelRecord(String name){
            this.levelname = name;
            // init new empty record
            for(int c=0; c<10; c++){
                record[c] = new Item("", 0);
            }
        }

        public void addItem(String name, int score){
            // sort record item's
            for(int c=0; c<10; c++){
                if(record[c].score < score){
                    for(int i=9; i>c; i--){
                        record[i].name = record[i-1].name;
                        record[i].score = record[i-1].score;
                    }
                    record[c] = new Item(name, score);
                    break;
                }
            }
        }
    }
    private class Item{
        public String name;
        public int score;

        public Item(String name, int score){
            this.name = name;
            this.score = score;
        }
    }

    // other
    public void draw(Graphics g){
        g.setFont(first);
        g.setColor(Color.black);
        g.drawString("High scores", Const.WIDTH/2-fmetr.stringWidth("High scores")/2, 70);
        it = table.iterator();
        while(it.hasNext()){
            pointer = (LevelRecord)it.next();
            if(pointer.levelname.equals(current.levelname)){
                g.setFont(second);
                g.drawString(current.levelname, Const.WIDTH/2-smetr.stringWidth(current.levelname)/2, 110);
                if(left) g.setColor(Color.red);
                g.drawString("<<", Const.WIDTH/2-smetr.stringWidth(current.levelname)/2-50, 110);
                if(left && !right) g.setColor(Color.black);
                if(!left && right) g.setColor(Color.red);
                g.drawString(">>", Const.WIDTH/2+smetr.stringWidth(current.levelname)/2+20, 110);
                if(right) g.setColor(Color.black);
                for(int c=0; c<10; c++){
                    g.drawString(pointer.record[c].name, 150, 140+c*30);
                    g.drawString(""+pointer.record[c].score,
                            Const.WIDTH-150-smetr.stringWidth(""+pointer.record[c].score), 140+c*30);
                }
            }
        }
    }

    private void loadScores(String filename){
        String rname, pname;
        Scanner file;
        int c, pscore;
        // try to load, load if successfully, else error
        try {
            file = new Scanner(new File(filename));
            while(file.hasNextLine()){
                rname = file.nextLine();
                addRecord(rname);
                for(c=0; c<10; c++){
                    pname = file.nextLine();
                    pscore = Integer.parseInt(file.nextLine());
                    addItem(rname, pname, pscore);
                }
            }
            file.close();
        } catch (FileNotFoundException e) {
            System.out.print("Error loading high scores!\n");
            addRecord("Random");
        }
    }
    public void saveScores(String filename){
        try{
            FileWriter fw = new FileWriter(filename, false);
            it = table.iterator();
            while(it.hasNext()){
                pointer = (LevelRecord)it.next();
                fw.write(pointer.levelname+"\n");
                for(int c=0; c<10; c++){
                    fw.write(pointer.record[c].name+"\n");
                    fw.write(pointer.record[c].score+"\n");
                }
            }
            fw.close();
        } catch(IOException e){
            System.out.print("Error saving high scores!\n");
        }
    }

    public void mouseClicked(MouseEvent e){
        if(right){
            if(!currit.hasNext()){ currit = table.listIterator(); }
            current = (LevelRecord)currit.next();
        }
        else if(left){
            if(currit.nextIndex() == 0){
                while(currit.hasNext()){ current = (LevelRecord)currit.next(); }
            }
            else current = (LevelRecord)currit.previous();
        }
    }
    public void mouseMoved(MouseEvent e){
        left = leftRect.contains(e.getPoint());
        right = rightRect.contains(e.getPoint());
    }
}